unit hiQTimer;

interface

uses Kol,windows,objects,
     Share,Debug,Messages;

type
  TCallbackTimer = procedure(lpParam: Pointer;
                             TimerOrWaitFired: Boolean); stdcall;

  function CreateTimerQueueTimer(var Timer: THandle;
                                 TimerQueue: THandle;
                                 Callback: TCallbackTimer;
                                 Parameter: Pointer;
                                 DueTime: LongWord;
                                 Period: LongWord;
                                 Flags: LongWord): BOOL; stdcall;

  function DeleteTimerQueueTimer(TimerQueue: THandle;
                                 Timer: THandle;
                                 CompletionEvent: THandle): BOOL; stdcall;


  function  ChangeTimerQueueTimer(
    TimerQueue: THandle;   //    ( 0   )
    Timer: THandle;        //  
    DueTime: LongWord;     //     
    Period: LongWord       //    
  ): BOOL; stdcall;


type
  THIQTimer = class(TDebug)
   private
    AutoStop: integer;

    bTimerEnabled: Boolean;
    hTimerHandle: THandle;
    hwWindowHandle: HWND;

    uiDueTime: LongWord;
    uiPeriod: LongWord;
    uiThreadCount: LongWord;
    uiCount: LongWord;

    procedure TimerWndProc(var Msg: TMessage);
    procedure SetInterval(Value:integer);
    procedure SetEnable(Value:boolean);
    procedure SetAutoStop(Value:integer);
    procedure OnStop;
    procedure OnTimer;

   public
    _event_onTimer:THI_Event;
    _event_onStop:THI_Event;
    constructor Create;
    destructor Destroy; override;
    procedure _work_doTimer(var _Data:TData; Index:word);
    procedure _work_doStop(var _Data:TData; Index:word);
    procedure _work_doInterval(var _Data:TData; Index:word);
    procedure _work_doAutoStop(var _Data:TData; Index:word);
    property  _prop_Interval:integer write SetInterval;
    property  _prop_Enable:boolean write SetEnable;
    property  _prop_AutoStop:integer write SetAutoStop;

  end;

implementation

const
  Kernel32Lib = 'kernel32.dll';

  function CreateTimerQueueTimer; external Kernel32Lib name 'CreateTimerQueueTimer';
  function DeleteTimerQueueTimer; external Kernel32Lib name 'DeleteTimerQueueTimer';
  function ChangeTimerQueueTimer; external Kernel32Lib name 'ChangeTimerQueueTimer';

procedure TimerCallback(Timer: THIQTimer; CallbackFired: Boolean); stdcall;
begin
  Inc(Timer.uiThreadCount);
  PostMessage(Timer.hwWindowHandle, WM_APP + 1,0,Timer.uiThreadCount);
end;

constructor THIQTimer.Create;
begin
  inherited Create;
  //uiDueTime := 1000;
  //uiPeriod := 1000;
  hwWindowHandle := AllocateHWnd(TimerWndProc);
end;

destructor THIQTimer.Destroy;
begin
  OnStop;
  DeallocateHWnd(hwWindowHandle);
  inherited Destroy;
end;

procedure THIQTimer.TimerWndProc(var Msg: TMessage);
begin
  if Msg.Msg = (WM_APP + 1) then
    begin
      uiCount:= Msg.lParam;
      OnTimer;
    end
  else
    msg.Result := DefWindowProc(hwWindowHandle,Msg.Msg,Msg.wParam,Msg.lParam);
end;

procedure THIQTimer._work_doTimer;
begin
  SetEnable(True);
end;

procedure THIQTimer._work_doStop;
begin
  if bTimerEnabled then
    OnStop;
end;

procedure THIQTimer._work_doInterval;
begin
  uiDueTime := ToInteger(_Data);
  uiPeriod := ToInteger(_Data);
  if (bTimerEnabled = True) then
    ChangeTimerQueueTimer(0,hTimerHandle,uiDueTime,uiPeriod);
end;

procedure THIQTimer._work_doAutoStop;
begin
  AutoStop := ToInteger(_Data);
end;

procedure THIQTimer.SetInterval;
begin
  uiDueTime := Value;
  uiPeriod := Value;
end;

procedure THIQTimer.SetEnable;
begin
  if (Value <> bTimerEnabled) then
    begin
      if Value then
        begin
          uiThreadCount := 0;
          uiCount := 0;
          bTimerEnabled := CreateTimerQueueTimer(hTimerHandle,0,@TimerCallback,
                             Self,uiDueTime,uiPeriod,ULONG($00000000));
        end
      else
        begin
          DeleteTimerQueueTimer(0,hTimerHandle,0{INVALID_HANDLE_VALUE});
          bTimerEnabled := False;
        end;
    end;
end;

procedure THIQTimer.SetAutoStop;
begin
  AutoStop := Value;
end;

procedure THIQTimer.OnTimer;
var f: boolean;
begin
  if AutoStop = 0 then
  begin
    _hi_OnEvent(_event_onTimer);
    Exit;
  end;

  if AutoStop >= uiCount then
    begin
      f := AutoStop = uiCount;
      _hi_OnEvent(_event_onTimer);
    end;
  if f then OnStop;
end;

procedure THIQTimer.OnStop;
begin
  _hi_OnEvent(_event_onStop);
  SetEnable(False);
end;

end.
